# 9p protcol notes

## Man pages
  online: http://man.cat-v.org/plan_9/5/
  local html: ~/repos/personal/plan9port/man/man9/index.html
  local man: `9 man 9 intro`

## Other references
  https://www.kernel.org/doc/Documentation/filesystems/9p.txt
  http://9p.cat-v.org/documentation/
  https://github.com/9fans/go/blob/main/cmd/acme/fsys1.go

## Overview
The man pages for the protocol are pretty clear on how things are supposed to work:

INTRO(9p)

  Each message consists of a sequence of bytes. Two-, four-, and eight-byte fields hold unsigned integers represented
  in little-endian order (least significant byte first). Data items of larger or variable lengths are represented by
  a two-byte field specifying a count, n, followed by n bytes of data. Text strings are represented this way, with the
  text itself stored as a UTF-8 encoded sequence of Unicode characters (see utf(7)). Text strings in 9P messages are
  not NUL-terminated: n counts the bytes of UTF-8 data, which include no final zero byte. The NUL character is illegal
  in all text strings in 9P, and is therefore excluded from file names, user names, and so on.

  Each 9P message begins with a four-byte size field specifying the length in bytes of the complete message including
  the four bytes of the size field itself. The next byte is the message type, one of the constants in the enumeration
  in the include file <fcall.h>. The next two bytes are an identifying tag, described below. The remaining bytes are
  parameters of different sizes. In the message descriptions, the number of bytes in a field is given in brackets after
  the field name. The notation parameter[n] where n is not a constant represents a variable-length parameter: n[2]
  followed by n bytes of data forming the parameter. The notation string[s] (using a literal s character) is shorthand
  for s[2] followed by s bytes of UTF-8 text. (Systems may choose to reduce the set of legal characters to reduce
  syntactic problems, for example to remove slashes from name components, but the protocol has no such restriction.
  Plan 9 names may contain any printable character (that is, any character outside hexadecimal 00-1F and 80-9F) except
  slash.) Messages are transported in byte form to allow for machine independence

The rest of the man pages go into the specifics of each message type and how the data for
each is handled.


### The enum from <fcall.h>
```c
enum
{
	Tversion =	100,
	Rversion,
	Tauth =		102,
	Rauth,
	Tattach =	104,
	Rattach,
	Terror =	106,	/* illegal */
	Rerror,
	Tflush =	108,
	Rflush,
	Twalk =		110,
	Rwalk,
	Topen =		112,
	Ropen,
	Tcreate =	114,
	Rcreate,
	Tread =		116,
	Rread,
	Twrite =	118,
	Rwrite,
	Tclunk =	120,
	Rclunk,
	Tremove =	122,
	Rremove,
	Tstat =		124,
	Rstat,
	Twstat =	126,
	Rwstat,
	Tmax,

	Topenfd = 	98,
	Ropenfd
};
```


## Message types

The version message identifies the version of the protocol and indicates the maximum message size the system is prepared
to handle. It also initializes the connection and aborts all outstanding I/O on the connection. The set of messages
between version requests is called a session. 

    size[4] Tversion tag[2] msize[4] version[s]
    size[4] Rversion tag[2] msize[4] version[s]


    size[4] Tauth tag[2] afid[4] uname[s] aname[s]
    size[4] Rauth tag[2] aqid[13]

    size[4] Rerror tag[2] ename[s]

    size[4] Tflush tag[2] oldtag[2]
    size[4] Rflush tag[2]

    size[4] Tattach tag[2] fid[4] afid[4] uname[s] aname[s]
    size[4] Rattach tag[2] qid[13]

    size[4] Twalk tag[2] fid[4] newfid[4] nwname[2] nwname*(wname[s])
    size[4] Rwalk tag[2] nwqid[2] nwqid*(wqid[13])

    size[4] Topen tag[2] fid[4] mode[1]
    size[4] Ropen tag[2] qid[13] iounit[4]

    size[4] Topenfd tag[2] fid[4] mode[1]
    size[4] Ropenfd tag[2] qid[13] iounit[4] unixfd[4]

    size[4] Tcreate tag[2] fid[4] name[s] perm[4] mode[1]
    size[4] Rcreate tag[2] qid[13] iounit[4]

    size[4] Tread tag[2] fid[4] offset[8] count[4]
    size[4] Rread tag[2] count[4] data[count]

    size[4] Twrite tag[2] fid[4] offset[8] count[4] data[count]
    size[4] Rwrite tag[2] count[4]

    size[4] Tclunk tag[2] fid[4]
    size[4] Rclunk tag[2]

    size[4] Tremove tag[2] fid[4]
    size[4] Rremove tag[2]

    size[4] Tstat tag[2] fid[4]
    size[4] Rstat tag[2] stat[n]

    size[4] Twstat tag[2] fid[4] stat[n]
    size[4] Rwstat tag[2]
